"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processInBatches = processInBatches;
exports.retryWithBackoff = retryWithBackoff;
async function processInBatches(processBatch, handler, logger, options) {
    const batchSize = options?.batchSize ?? 1000;
    const maxConcurrentBatches = 3;
    logger.info({ batchSize, maxConcurrentBatches }, "Starting batch processing.");
    let offset = 0;
    let totalItemsProcessed = 0;
    let batchCounter = 0;
    let activeBatches = [];
    let processingComplete = false;
    while (!processingComplete) {
        let items;
        try {
            logger.trace({ offset, batchSize, batchNum: batchCounter + 1 }, "Fetching next batch.");
            items = await processBatch(offset, batchSize);
        }
        catch (fetchError) {
            logger.error({ err: fetchError, offset, batchSize }, "Error fetching batch from processBatch.");
            throw fetchError;
        }
        if (!items || items.length === 0) {
            logger.debug({ offset, batchSize }, "Received empty batch, signalling end of data.");
            processingComplete = true;
            break;
        }
        batchCounter++;
        const currentBatchNum = batchCounter;
        const itemCount = items.length;
        totalItemsProcessed += itemCount;
        logger.debug({ itemCount, batchNum: currentBatchNum, offset }, `Processing batch ${currentBatchNum}.`);
        const batchTask = handler(items)
            .then(() => {
            logger.trace({ batchNum: currentBatchNum }, `Handler finished for batch ${currentBatchNum}.`);
        })
            .catch((handlerError) => {
            logger.error({ err: handlerError, batchNum: currentBatchNum }, `Error processing batch ${currentBatchNum} in handler.`);
            throw handlerError;
        });
        activeBatches.push(batchTask);
        if (activeBatches.length >= maxConcurrentBatches) {
            logger.trace({ activeCount: activeBatches.length }, `Concurrency limit reached, waiting for batches to complete...`);
            try {
                await Promise.all(activeBatches);
            }
            catch (error) {
                logger.error({ err: error }, `Error occurred while waiting for concurrent batch handlers. Stopping.`);
                processingComplete = true;
                break;
            }
            logger.trace("Concurrency slot freed.");
            activeBatches = [];
        }
        offset += itemCount;
    }
    if (activeBatches.length > 0) {
        logger.debug({ remainingCount: activeBatches.length }, `Waiting for ${activeBatches.length} final batch(es) to complete...`);
        try {
            await Promise.all(activeBatches);
        }
        catch (finalError) {
            logger.error({ err: finalError }, `Error occurred while waiting for final batch handlers.`);
            throw finalError;
        }
    }
    logger.info({ totalItemsProcessed, batchCount: batchCounter }, "Batch processing finished.");
}
async function retryWithBackoff(operation, logger, operationName = "Unnamed Operation", maxRetries = 5, baseDelay = 500) {
    let attempt = 0;
    logger.trace({ attempt: attempt + 1, maxRetries, operationName }, `Attempting operation.`);
    while (attempt <= maxRetries) {
        try {
            const result = await operation();
            if (attempt > 0) {
                logger.debug({ attempt: attempt + 1, operationName }, `Operation succeeded after retry.`);
            }
            return result;
        }
        catch (error) {
            const isLastError = attempt === maxRetries;
            const logContext = {
                err: error,
                attempt: attempt + 1,
                maxRetries,
                operationName,
            };
            if (isLastError) {
                logger.error(logContext, `Max retries reached. Operation failed.`);
                throw error;
            }
            const delay = baseDelay * Math.pow(2, attempt);
            logger.warn({ ...logContext, delay }, `Operation failed. Retrying after ${delay}ms...`);
            await new Promise((resolve) => setTimeout(resolve, delay));
            attempt++;
        }
    }
    const finalError = new Error(`${operationName} failed after ${maxRetries + 1} attempts.`);
    logger.error({ operationName, maxRetries }, finalError.message);
    throw finalError;
}
//# sourceMappingURL=BatchProcessor.js.map