"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.batchSender = void 0;
exports.batchSender = {
    async process(recipients, meta, rateLimiter, sendSingle, logger, options) {
        const concurrency = options?.concurrency ?? 5;
        const results = [];
        const tasks = [];
        let successCount = 0;
        let failureCount = 0;
        let skippedCount = 0;
        logger.info({ recipientCount: recipients.length, concurrency }, "Starting batch processing via BatchSender.");
        for (let i = 0; i < recipients.length; i++) {
            const recipient = recipients[i];
            const userMeta = meta?.[i];
            const taskLogger = logger.child({
                recipient: String(recipient)?.slice(-10),
                index: i,
            });
            if (!recipient) {
                taskLogger.warn({ validationRule: "missingOrEmptyRecipient" }, "Skipping invalid recipient (missing or empty).");
                results[i] = {
                    status: "error",
                    recipient: `invalid_recipient_at_index_${i}`,
                    error: "Invalid recipient data",
                };
                skippedCount++;
                continue;
            }
            if (!userMeta) {
                taskLogger.warn({ validationRule: "missingMeta" }, "Skipping recipient due to missing meta.");
                results[i] = {
                    status: "error",
                    recipient: String(recipient),
                    error: "Missing meta for recipient",
                };
                skippedCount++;
                continue;
            }
            taskLogger.setBindings({ recipient: String(recipient).slice(-10) });
            const currentIndex = i;
            const task = rateLimiter.schedule(async () => {
                taskLogger.trace("Executing sendSingle via rate limiter.");
                let result = null;
                try {
                    result = await sendSingle(recipient, userMeta, taskLogger);
                }
                catch (error) {
                    let errorMessage = "Unknown error during sendSingle execution";
                    if (error instanceof Error)
                        errorMessage = error.message;
                    taskLogger.error({ err: error }, "Unexpected error executing sendSingle function.");
                    result = {
                        status: "error",
                        recipient: String(recipient),
                        error: "INTERNAL_SEND_ERROR",
                        response: errorMessage,
                    };
                }
                if (result) {
                    results[currentIndex] = result;
                    if (result.status === "success") {
                        successCount++;
                    }
                    else {
                        failureCount++;
                    }
                }
                else {
                    results[currentIndex] = {
                        status: "error",
                        recipient: String(recipient),
                        error: "UNKNOWN_PROCESSING_FAILURE",
                    };
                    failureCount++;
                }
            });
            tasks.push(task);
            if (tasks.length >= concurrency) {
                logger.trace({ taskCount: tasks.length }, `Concurrency limit reached, awaiting batch...`);
                try {
                    await Promise.all(tasks);
                }
                catch (e) {
                    logger.error({ err: e }, "Error during concurrent batch wait.");
                }
                tasks.length = 0;
            }
        }
        if (tasks.length > 0) {
            logger.trace({ taskCount: tasks.length }, `Awaiting final ${tasks.length} task(s)...`);
            try {
                await Promise.all(tasks);
            }
            catch (e) {
                logger.error({ err: e }, "Error during final batch wait.");
            }
        }
        const finalResults = recipients.map((_, idx) => results[idx] || {
            status: "error",
            recipient: String(recipients[idx] || `unknown_at_index_${idx}`),
            error: "PROCESSING_ERROR_OR_SKIPPED",
        });
        logger.info({
            successCount,
            failureCount,
            skippedCount,
            totalAttempted: recipients.length,
        }, "Finished batch processing via BatchSender.");
        return finalResults;
    },
};
//# sourceMappingURL=BatchSender.js.map