"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenBucketRateLimiter = exports.RateLimiter = void 0;
const bottleneck_1 = __importDefault(require("bottleneck"));
class RateLimiter {
    limiter;
    constructor(maxRequests, perMilliseconds) {
        this.limiter = new bottleneck_1.default({
            maxConcurrent: maxRequests,
            minTime: perMilliseconds / maxRequests,
        });
    }
    schedule(fn, ...args) {
        return this.limiter.schedule(() => fn(...args));
    }
}
exports.RateLimiter = RateLimiter;
class TokenBucketRateLimiter {
    tokens;
    capacity;
    refillRate;
    lastRefillTimestamp;
    logger;
    constructor(maxRatePerSecond, logger) {
        if (maxRatePerSecond <= 0) {
            throw new Error("TokenBucketRateLimiter maxRatePerSecond must be positive.");
        }
        this.capacity = maxRatePerSecond;
        this.tokens = this.capacity;
        this.refillRate = maxRatePerSecond / 1000;
        this.lastRefillTimestamp = Date.now();
        this.logger = logger?.child({ component: "TokenBucketRateLimiter" });
        this.logger?.debug({ capacity: this.capacity, refillRate: this.refillRate }, "Initialized.");
    }
    refillTokens() {
        const now = Date.now();
        const elapsedTime = now - this.lastRefillTimestamp;
        if (elapsedTime <= 0) {
            this.logger?.trace("No time elapsed since last refill.");
            return;
        }
        const newTokens = elapsedTime * this.refillRate;
        const previousTokens = this.tokens;
        this.tokens = Math.min(this.capacity, this.tokens + newTokens);
        this.lastRefillTimestamp = now;
        this.logger?.trace({ elapsedTime, newTokens, previousTokens, currentTokens: this.tokens }, "Tokens refilled.");
    }
    async acquire() {
        this.refillTokens();
        if (this.tokens >= 1) {
            this.tokens -= 1;
            this.logger?.trace({ tokensRemaining: this.tokens }, "Token acquired immediately.");
            return;
        }
        const waitStart = Date.now();
        this.logger?.debug({ tokensAvailable: this.tokens }, "Waiting for token...");
        while (this.tokens < 1) {
            const tokensNeeded = 1 - this.tokens;
            const estimatedWaitMs = Math.ceil(tokensNeeded / this.refillRate);
            const waitTime = Math.max(10, Math.min(estimatedWaitMs / 2, 50));
            await new Promise((resolve) => setTimeout(resolve, waitTime));
            this.refillTokens();
        }
        const waitEnd = Date.now();
        this.tokens -= 1;
        this.logger?.debug({ waitedMs: waitEnd - waitStart, tokensRemaining: this.tokens }, "Token acquired after waiting.");
    }
}
exports.TokenBucketRateLimiter = TokenBucketRateLimiter;
//# sourceMappingURL=RateLimiter.js.map