"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmailNotifier = void 0;
const nodemailer_1 = __importDefault(require("nodemailer"));
const RateLimiter_1 = require("../../core/RateLimiter");
const LoggerFactory_1 = require("../../utils/LoggerFactory");
const BatchSender_1 = require("../../core/BatchSender");
class EmailNotifier {
    transporter;
    rateLimiter;
    config;
    baseLogger;
    constructor(config) {
        this.config = config;
        this.baseLogger = LoggerFactory_1.loggerFactory.createLogger({
            component: "EmailNotifier",
        });
        this.baseLogger.info({ host: config.host, port: config.port, user: config.auth.user }, "Initializing...");
        try {
            this.transporter = nodemailer_1.default.createTransport({
                host: config.host,
                port: config.port,
                secure: config.secure,
                auth: config.auth,
                pool: true,
                maxConnections: 5,
            });
        }
        catch (err) {
            let errorMessage = "An unknown error occurred";
            if (err instanceof Error) {
                errorMessage = err.message;
            }
            else if (typeof err === "string") {
                errorMessage = err;
            }
            throw new Error(`Failed to create Nodemailer transporter: ${errorMessage}`);
        }
        const maxEmailsPerSecond = config.maxEmailsPerSecond || 10;
        this.rateLimiter = new RateLimiter_1.RateLimiter(maxEmailsPerSecond, 1000);
    }
    async send(emails, meta, logger) {
        return BatchSender_1.batchSender.process(emails, meta, this.rateLimiter, this._sendSingleEmail.bind(this), logger, { concurrency: 3 });
    }
    async _sendSingleEmail(email, emailMeta, taskLogger) {
        if (!emailMeta || !emailMeta.subject) {
            taskLogger.warn("Missing subject in email meta.");
            return { status: "error", recipient: email, error: "MISSING_SUBJECT" };
        }
        try {
            const emailOptions = {
                from: this.config.from,
                to: email,
                subject: emailMeta.subject,
                ...(emailMeta.html
                    ? { html: emailMeta.html }
                    : { text: emailMeta.text || "" }),
                ...(emailMeta.attachments
                    ? { attachments: emailMeta.attachments }
                    : {}),
            };
            taskLogger.trace({ options: { subject: emailOptions.subject } }, "Sending email via transporter...");
            const info = await this.transporter.sendMail(emailOptions);
            taskLogger.debug({ messageId: info.messageId, accepted: info.accepted?.length }, "Email sent successfully.");
            return {
                status: "success",
                recipient: email,
                response: {
                    messageId: info.messageId,
                    accepted: info.accepted,
                    rejected: info.rejected,
                },
            };
        }
        catch (err) {
            let errorMessage = "Unknown email send error";
            let statusCode = "N/A";
            let nodeCode;
            if (err instanceof Error) {
                errorMessage = err.message;
                nodeCode = err.code;
                statusCode = err.responseCode || "N/A";
            }
            taskLogger.warn({ err, errorCode: nodeCode || statusCode }, `Failed sending email.`);
            const sanitizedMessage = errorMessage
                .replace(/\s+/g, "_")
                .replace(/[.:;,*+?^${}()|[\]\\]/g, "");
            const primaryCode = statusCode !== "N/A" ? statusCode : nodeCode || "N/A";
            const errorKeyBody = `${primaryCode}:${sanitizedMessage}`;
            return {
                status: "error",
                recipient: email,
                error: errorKeyBody.substring(0, 255),
                response: {
                    message: errorMessage,
                    code: nodeCode,
                    statusCode: statusCode,
                },
            };
        }
    }
    async close(logger) {
        if (this.transporter && this.transporter.close) {
            logger.info("Closing Nodemailer transporter connections...");
            this.transporter.close();
        }
    }
}
exports.EmailNotifier = EmailNotifier;
//# sourceMappingURL=EmailNotifier.js.map