"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseNotifier = void 0;
const firebase_admin_1 = __importDefault(require("firebase-admin"));
const LoggerFactory_1 = require("../../utils/LoggerFactory");
const RateLimiter_1 = require("../../core/RateLimiter");
const BatchSender_1 = require("../../core/BatchSender");
class FirebaseNotifier {
    static initialized = false;
    rateLimiter;
    credentialInfo;
    app;
    baseLogger;
    constructor(credentialInfo, config) {
        this.baseLogger = LoggerFactory_1.loggerFactory.createLogger({
            component: "FirebaseNotifier",
        });
        this.baseLogger.info("Initializing...");
        this.credentialInfo = credentialInfo;
        this.initFirebase(this.baseLogger);
        const maxMessagesPerSecond = config?.maxMessagesPerSecond || 500;
        this.rateLimiter = new RateLimiter_1.RateLimiter(maxMessagesPerSecond, 1000);
        this.baseLogger.info({ rateLimit: maxMessagesPerSecond }, "Rate limiter configured.");
    }
    initFirebase(logger) {
        if (FirebaseNotifier.initialized) {
            if (!this.app) {
                logger.debug("Attaching to already initialized Firebase Admin SDK instance.");
                this.app = firebase_admin_1.default.app();
            }
            else {
                logger.trace("Firebase already initialized, app instance already attached.");
            }
            return;
        }
        logger.debug("Performing Firebase Admin SDK initialization check.");
        if (firebase_admin_1.default.apps.length > 0) {
            this.app = firebase_admin_1.default.app();
            logger.info("Using existing Firebase Admin SDK instance (initialized externally).");
            FirebaseNotifier.initialized = true;
            return;
        }
        let credential;
        const credentialType = typeof this.credentialInfo === "string" ? "path" : "object";
        logger.debug({ credentialType }, "Attempting to load credentials...");
        try {
            if (typeof this.credentialInfo === "string") {
                credential = firebase_admin_1.default.credential.cert(this.credentialInfo);
                logger.debug("Credentials loaded from path.");
            }
            else if (typeof this.credentialInfo === "object" &&
                this.credentialInfo !== null) {
                if (!this.credentialInfo.projectId ||
                    !this.credentialInfo.privateKey ||
                    !this.credentialInfo.clientEmail) {
                    throw new Error("ServiceAccount object is missing required properties (projectId, privateKey, clientEmail).");
                }
                credential = firebase_admin_1.default.credential.cert(this.credentialInfo);
                logger.debug("Credentials loaded from object.");
            }
            else {
                throw new Error("Invalid Firebase credential format.");
            }
        }
        catch (e) {
            let errorMessage = "An unknown credential error occurred";
            if (e instanceof Error)
                errorMessage = e.message;
            logger.error({ err: e, credentialType }, `Failed to load Firebase credentials.`);
            throw new Error(`Invalid Firebase credential configuration: ${errorMessage}`);
        }
        try {
            logger.debug("Initializing Firebase Admin SDK app...");
            this.app = firebase_admin_1.default.initializeApp({ credential });
            logger.info("Firebase Admin SDK initialized successfully.");
            FirebaseNotifier.initialized = true;
        }
        catch (initError) {
            let errorMessage = "An unknown initialization error occurred";
            if (initError instanceof Error)
                errorMessage = initError.message;
            logger.error({ err: initError }, "Failed to initialize Firebase Admin SDK.");
            throw new Error(`Firebase Admin SDK initialization failed: ${errorMessage}`);
        }
    }
    async send(tokens, meta, logger) {
        if (!this.app) {
            logger.error("Firebase SDK App instance is not available. Cannot send.");
            return tokens.map((token) => ({
                status: "error",
                recipient: token,
                error: "Firebase SDK not initialized properly",
            }));
        }
        return BatchSender_1.batchSender.process(tokens, meta, this.rateLimiter, this._sendSingleFCM.bind(this), logger, { concurrency: 5 });
    }
    async _sendSingleFCM(token, userMeta, taskLogger) {
        if (!this.app) {
            taskLogger.error("Firebase app instance missing in _sendSingleFCM.");
            return { status: "error", recipient: token, error: "INTERNAL_SDK_ERROR" };
        }
        const messaging = this.app.messaging();
        try {
            if (!userMeta || typeof userMeta !== "object") {
                taskLogger.warn("Invalid meta object provided for single send.");
                return {
                    status: "error",
                    recipient: token,
                    error: "Invalid meta for recipient",
                };
            }
            const messageToSend = {
                token: token,
                ...(userMeta.notification
                    ? { notification: userMeta.notification }
                    : {
                        notification: { title: userMeta.title, body: userMeta.body },
                    }),
                ...(userMeta.android ? { android: userMeta.android } : {}),
                ...(userMeta.apns ? { apns: userMeta.apns } : {}),
                ...(userMeta.webpush ? { webpush: userMeta.webpush } : {}),
                ...(userMeta.fcmOptions ? { fcmOptions: userMeta.fcmOptions } : {}),
                ...(userMeta.data ? { data: userMeta.data } : {}),
            };
            if (messageToSend.notification &&
                !messageToSend.notification.title &&
                !messageToSend.notification.body) {
                taskLogger.trace("Removing empty notification object from payload.");
                delete messageToSend.notification;
            }
            if (!messageToSend.notification &&
                (!messageToSend.data || Object.keys(messageToSend.data).length === 0)) {
                taskLogger.error("Cannot send message without notification or data payload.");
                return {
                    status: "error",
                    recipient: token,
                    error: "INVALID_PAYLOAD",
                    response: "Message must contain notification or data",
                };
            }
            taskLogger.trace("Sending message via FCM...");
            const response = await messaging.send(messageToSend);
            taskLogger.debug({ messageId: response }, "Firebase message sent successfully.");
            return {
                status: "success",
                recipient: token,
                response: response,
            };
        }
        catch (error) {
            let errorMessage = "Unknown FCM send error";
            let firebaseCode = "UNKNOWN_FCM_ERROR";
            let statusCode = "N/A";
            if (error instanceof Error) {
                errorMessage = error.message;
                if (error.code && typeof error.code === "string") {
                    firebaseCode = error.code;
                }
            }
            taskLogger.warn({ err: error, firebaseCode }, `Firebase send error for token.`);
            const sanitizedMessage = errorMessage
                .replace(/\s+/g, "_")
                .replace(/[.:;,*+?^${}()|[\]\\]/g, "");
            const errorKeyBody = `${statusCode}:${firebaseCode}:${sanitizedMessage}`;
            return {
                status: "error",
                recipient: token,
                error: errorKeyBody.substring(0, 255),
                response: {
                    message: errorMessage,
                    code: firebaseCode,
                },
            };
        }
    }
}
exports.FirebaseNotifier = FirebaseNotifier;
//# sourceMappingURL=FirebaseNotifier.js.map