"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TelegramNotifier = void 0;
const telegraf_1 = require("telegraf");
const LoggerFactory_1 = require("../../utils/LoggerFactory");
const RateLimiter_1 = require("../../core/RateLimiter");
const BatchSender_1 = require("../../core/BatchSender");
class TelegramNotifier {
    config;
    bot;
    rateLimiter;
    baseLogger;
    constructor(config) {
        this.config = config;
        this.baseLogger = LoggerFactory_1.loggerFactory.createLogger({
            component: "TelegramNotifier",
        });
        this.baseLogger.info("Initializing...");
        if (!config.botToken) {
            const err = new Error("Telegram botToken is required.");
            this.baseLogger.error({ err }, "Initialization failed.");
            throw err;
        }
        try {
            this.bot = new telegraf_1.Telegraf(config.botToken);
            this.baseLogger.info("Telegraf instance created.");
        }
        catch (err) {
            this.baseLogger.error({ err }, "Failed to create Telegraf instance.");
            throw err;
        }
        const maxMessagesPerSecond = config.maxMessagesPerSecond || 25;
        this.rateLimiter = new RateLimiter_1.RateLimiter(maxMessagesPerSecond, 1000);
        this.baseLogger.info({ rateLimit: maxMessagesPerSecond }, "Rate limiter configured.");
    }
    async send(chatIds, meta, logger) {
        return BatchSender_1.batchSender.process(chatIds, meta, this.rateLimiter, this._sendSingleTelegramMessage.bind(this), logger, { concurrency: 5 });
    }
    async _sendSingleTelegramMessage(chatId, userMeta, taskLogger) {
        if (!userMeta || !userMeta.text) {
            taskLogger.warn("Missing text in Telegram meta.");
            return { status: "error", recipient: chatId, error: "MISSING_TEXT" };
        }
        const messageText = userMeta.text;
        const { text, ...sendOptions } = userMeta;
        const finalSendOptions = {
            parse_mode: "HTML",
            ...sendOptions,
        };
        try {
            taskLogger.trace({ options: finalSendOptions }, "Sending message via Telegraf...");
            const response = await this.bot.telegram.sendMessage(chatId, messageText, finalSendOptions);
            taskLogger.debug({ messageId: response.message_id }, "Telegram message sent successfully.");
            return {
                status: "success",
                recipient: chatId,
                response: response,
            };
        }
        catch (error) {
            let errorMessage = error.description || error.message || "Unknown Telegram error";
            let statusCode = error.code || error.response?.error_code || "N/A";
            taskLogger.warn({ err: error, statusCode, description: error.description }, `Telegram send error.`);
            const sanitizedMessage = errorMessage
                .replace(/\s+/g, "_")
                .replace(/[.:;,*+?^${}()|[\]\\]/g, "");
            const errorKeyBody = `${statusCode}:${sanitizedMessage}`;
            return {
                status: "error",
                recipient: chatId,
                error: errorKeyBody.substring(0, 255),
                response: {
                    message: error.message,
                    description: error.description,
                    code: error.code,
                    responsePayload: error.response,
                },
            };
        }
    }
}
exports.TelegramNotifier = TelegramNotifier;
//# sourceMappingURL=TelegramNotifier.js.map