"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebPushNotifier = void 0;
const web_push_1 = __importStar(require("web-push"));
const LoggerFactory_1 = require("../../utils/LoggerFactory");
const RateLimiter_1 = require("../../core/RateLimiter");
const BatchSender_1 = require("../../core/BatchSender");
class WebPushNotifier {
    config;
    rateLimiter;
    baseLogger;
    constructor(config) {
        this.config = config;
        this.baseLogger = LoggerFactory_1.loggerFactory.createLogger({
            component: "WebPushNotifier",
        });
        this.baseLogger.info("Initializing...");
        if (!config.publicKey || !config.privateKey || !config.contactEmail) {
            const err = new Error("WebPushNotifier requires publicKey, privateKey, and contactEmail.");
            this.baseLogger.error({ err }, "Initialization failed due to missing VAPID details.");
            throw err;
        }
        try {
            web_push_1.default.setVapidDetails("mailto:" + this.config.contactEmail, this.config.publicKey, this.config.privateKey);
            this.baseLogger.info("VAPID details set successfully.");
        }
        catch (err) {
            this.baseLogger.error({ err }, "Failed to set VAPID details.");
            throw err;
        }
        const maxMessagesPerSecond = config.maxMessagesPerSecond || 50;
        this.rateLimiter = new RateLimiter_1.RateLimiter(maxMessagesPerSecond, 1000);
        this.baseLogger.info({ rateLimit: maxMessagesPerSecond }, "Rate limiter configured.");
    }
    async send(subscriptionStrings, meta, logger) {
        return BatchSender_1.batchSender.process(subscriptionStrings, meta, this.rateLimiter, this._sendSingleWebPush.bind(this), logger, { concurrency: 5 });
    }
    async _sendSingleWebPush(subString, userMeta, taskLogger) {
        let subscription;
        let endpoint = `unparseable_sub_at_index_${taskLogger.bindings().index ?? "?"}`;
        try {
            if (!subString || typeof subString !== "string") {
                throw new Error("Subscription string is missing or not a string.");
            }
            subscription = JSON.parse(subString);
            if (!subscription ||
                typeof subscription !== "object" ||
                !subscription.endpoint ||
                !subscription.keys?.p256dh ||
                !subscription.keys?.auth) {
                throw new Error("Parsed subscription object is invalid or missing required keys.");
            }
            endpoint = subscription.endpoint;
            taskLogger.setBindings({ recipientEndpoint: endpoint });
            taskLogger.trace("Subscription string parsed successfully.");
        }
        catch (parseError) {
            taskLogger.warn({ err: parseError, inputString: subString?.substring(0, 50) + "..." }, "Skipping invalid or unparseable subscription string.");
            return {
                status: "error",
                recipient: endpoint,
                error: "INVALID_SUBSCRIPTION_STRING",
                response: parseError.message,
            };
        }
        if (!userMeta || typeof userMeta !== "object") {
            taskLogger.warn("Skipping subscription due to missing or invalid meta object.");
            return {
                status: "error",
                recipient: endpoint,
                error: "INVALID_META",
                response: "Missing or invalid meta for recipient",
            };
        }
        if (!userMeta.title && !userMeta.body && !userMeta.data) {
            taskLogger.warn("Meta lacks title, body, and data. Sending potentially empty notification.");
            userMeta.title = userMeta.title || "Notification";
        }
        try {
            const pushPayload = JSON.stringify({
                title: userMeta.title,
                body: userMeta.body || "",
                icon: userMeta.icon,
                image: userMeta.image,
                badge: userMeta.badge,
                data: userMeta.data || {},
            });
            const requestOptions = {};
            if (userMeta.TTL !== undefined)
                requestOptions.TTL = userMeta.TTL;
            if (userMeta.headers)
                requestOptions.headers = userMeta.headers;
            taskLogger.trace({ options: requestOptions, payloadSize: pushPayload.length }, "Sending notification via web-push...");
            const response = await web_push_1.default.sendNotification(subscription, pushPayload, requestOptions);
            taskLogger.debug({ statusCode: response.statusCode }, "Web Push sent successfully.");
            return {
                status: "success",
                recipient: endpoint,
                response: {
                    statusCode: response.statusCode,
                    headers: response.headers,
                },
            };
        }
        catch (error) {
            let errorMessage = "Unknown web-push error";
            let statusCode = "N/A";
            if (error instanceof web_push_1.WebPushError) {
                errorMessage = error.body || error.message;
                statusCode = error.statusCode;
                taskLogger.warn({ err: error, statusCode }, `Web Push send failed (WebPushError).`);
            }
            else if (error instanceof Error) {
                errorMessage = error.message;
                taskLogger.warn({ err: error }, `Web Push send failed (Error).`);
            }
            else {
                try {
                    errorMessage = JSON.stringify(error);
                }
                catch { }
                taskLogger.warn({ error: error }, `Web Push send failed (Unknown error type).`);
            }
            const sanitizedMessage = errorMessage
                .replace(/\s+/g, "_")
                .replace(/[.:;,*+?^${}()|[\]\\]/g, "");
            const errorKeyBody = `${statusCode}:${sanitizedMessage}`;
            return {
                status: "error",
                recipient: endpoint,
                error: errorKeyBody.substring(0, 255),
                response: {
                    message: errorMessage,
                    statusCode: statusCode,
                    originalError: error,
                },
            };
        }
    }
}
exports.WebPushNotifier = WebPushNotifier;
//# sourceMappingURL=WebPushNotifier.js.map