"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dispatchNotifications = dispatchNotifications;
const ioredis_1 = __importDefault(require("ioredis"));
const BatchProcessor_1 = require("./core/BatchProcessor");
const QueueManager_1 = require("./core/QueueManager");
const LoggerFactory_1 = require("./utils/LoggerFactory");
const RateLimiter_1 = require("./core/RateLimiter");
const RedisHandler_1 = require("./utils/RedisHandler");
const dispatchLogger = LoggerFactory_1.loggerFactory.createLogger({
    component: "DispatchNotifications",
});
async function dispatchNotifications(options) {
    dispatchLogger.info({
        queue: options.queueName,
        jobName: options.jobName,
        channel: options.channelName,
        campaignId: options.campaignId,
    }, "Dispatch process starting...");
    let redisInstance;
    try {
        redisInstance = (0, RedisHandler_1.ensureRedisInstance)(options.redisConnection, dispatchLogger);
    }
    catch (error) {
        dispatchLogger.error({ err: error }, `Failed to initialize Redis connection.`);
        throw new Error(`Failed to initialize Redis: ${error.message}`);
    }
    const rateLimiter = options.maxQueriesPerSecond
        ? new RateLimiter_1.TokenBucketRateLimiter(options.maxQueriesPerSecond, dispatchLogger)
        : null;
    const enqueueMaxRetries = options.enqueueRetries ?? 3;
    const enqueueRetryBaseDelay = options.enqueueBaseDelay ?? 200;
    let totalRecordsProcessed = 0;
    try {
        await (0, BatchProcessor_1.processInBatches)(async (offset, limit) => {
            if (rateLimiter) {
                await rateLimiter.acquire();
            }
            return await (0, BatchProcessor_1.retryWithBackoff)(() => options.dbQuery(offset, limit), dispatchLogger, "DatabaseQuery");
        }, async (records) => {
            if (records.length === 0) {
                dispatchLogger.trace("Received empty batch from database, skipping enqueue.");
                return;
            }
            totalRecordsProcessed += records.length;
            const userIds = records.map(options.mapRecordToUserId);
            const jobPayload = {
                userIds,
                channel: options.channelName,
                meta: records.map((record) => {
                    try {
                        return options.meta(record);
                    }
                    catch (metaError) {
                        dispatchLogger.error({ err: metaError, record }, `Error generating meta for record`);
                        return {};
                    }
                }),
                trackResponses: options.trackResponses,
                trackingKey: options.trackingKey || "notifications:stats",
                campaignId: options.campaignId,
            };
            const finalJobOptions = {
                removeOnComplete: true,
                removeOnFail: false,
                ...(options.jobOptions || {}),
            };
            try {
                await (0, BatchProcessor_1.retryWithBackoff)(async () => {
                    await QueueManager_1.QueueManager.enqueueJob(redisInstance, options.queueName, options.jobName, jobPayload, dispatchLogger, finalJobOptions);
                }, dispatchLogger, "EnqueueJobBatch", enqueueMaxRetries, enqueueRetryBaseDelay);
            }
            catch (enqueueError) {
                throw enqueueError;
            }
        }, dispatchLogger, {
            batchSize: options.batchSize,
        });
    }
    catch (error) {
        throw error;
    }
    finally {
        if (!(options.redisConnection instanceof ioredis_1.default)) {
            dispatchLogger.debug({ status: redisInstance.status }, "Checking status of internally created Redis connection for cleanup.");
            if (redisInstance.status === "ready" ||
                redisInstance.status === "connecting") {
                try {
                    await redisInstance.quit();
                    dispatchLogger.info("Internally created Redis connection closed.");
                }
                catch (quitError) {
                    dispatchLogger.warn({ err: quitError }, "Error attempting to quit internally created Redis connection.");
                }
            }
            else {
                dispatchLogger.debug("Internally created Redis connection not in quittable state, skipping quit.");
            }
        }
        else {
            dispatchLogger.debug("External Redis connection provided, skipping cleanup.");
        }
        dispatchLogger.info("Dispatch process finished.");
    }
}
//# sourceMappingURL=runBatchNotificationProcessor.js.map