"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureRedisInstance = ensureRedisInstance;
const ioredis_1 = __importDefault(require("ioredis"));
function ensureRedisInstance(connection, logger) {
    const setupLogger = logger.child({ component: "RedisSetup" });
    if (connection instanceof ioredis_1.default) {
        setupLogger.debug("Validating provided Redis instance...");
        const options = connection.options;
        let isValid = true;
        if (options?.maxRetriesPerRequest !== null) {
            const errMsg = "Provided Redis instance must be configured with maxRetriesPerRequest: null for BullMQ compatibility.";
            setupLogger.error({ currentValue: options?.maxRetriesPerRequest }, errMsg);
            isValid = false;
            throw new Error(errMsg);
        }
        if (options?.enableReadyCheck !== false &&
            options?.enableReadyCheck !== undefined) {
            setupLogger.warn({ currentValue: options?.enableReadyCheck }, "Provided Redis instance does not have 'enableReadyCheck: false'. Recommended for BullMQ.");
        }
        if (isValid) {
            setupLogger.info("Provided Redis instance validated successfully.");
        }
        return connection;
    }
    if (typeof connection === "object" && connection !== null) {
        setupLogger.debug("Creating new Redis instance from options...");
        const optionsWithDefaults = {
            ...connection,
            maxRetriesPerRequest: null,
            enableReadyCheck: connection.enableReadyCheck === undefined
                ? false
                : connection.enableReadyCheck,
        };
        try {
            setupLogger.info({
                options: {
                    maxRetriesPerRequest: null,
                    enableReadyCheck: optionsWithDefaults.enableReadyCheck,
                },
            }, "Creating new Redis instance with BullMQ defaults.");
            const newInstance = new ioredis_1.default(optionsWithDefaults);
            setupLogger.debug("New Redis instance created successfully.");
            return newInstance;
        }
        catch (e) {
            setupLogger.error({ err: e, providedOptions: connection }, "Failed to create Redis instance from options.");
            throw new Error(`Failed to create Redis instance: ${e.message || e}`);
        }
    }
    const err = new Error("Invalid redisConnection provided: Must be an ioredis instance or RedisOptions object.");
    setupLogger.error({ providedType: typeof connection }, err.message);
    throw err;
}
//# sourceMappingURL=RedisHandler.js.map