"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startWorkerServer = startWorkerServer;
const NotifierRegistry_1 = require("../core/NotifierRegistry");
const TelegramNotifier_1 = require("../jobs/channels/TelegramNotifier");
const WorkerManager_1 = require("../core/WorkerManager");
const EmailNotifier_1 = require("../jobs/channels/EmailNotifier");
const FirebaseNotifier_1 = require("../jobs/channels/FirebaseNotifier");
const WebPushNotifier_1 = require("../jobs/channels/WebPushNotifier");
const ResponseTrackers_1 = require("./ResponseTrackers");
const RedisHandler_1 = require("./RedisHandler");
const LoggerFactory_1 = require("./LoggerFactory");
const notifierMap = {
    telegram: TelegramNotifier_1.TelegramNotifier,
    email: EmailNotifier_1.EmailNotifier,
    firebase: FirebaseNotifier_1.FirebaseNotifier,
    web: WebPushNotifier_1.WebPushNotifier,
};
function startWorkerServer(config) {
    const baseLogger = LoggerFactory_1.loggerFactory.createLogger({
        component: "StartWorkerServer",
        queue: config.queueName,
    });
    baseLogger.info("Starting worker server initialization...");
    if (!config.redisConnection) {
        const err = new Error("Missing 'redisConnection' in WorkerConfig");
        baseLogger.error({ err }, "Initialization failed.");
        throw err;
    }
    if (!config.queueName) {
        const err = new Error("Missing 'queueName' in WorkerConfig");
        baseLogger.error({ err }, "Initialization failed.");
        throw err;
    }
    baseLogger.debug("Ensuring Redis instance...");
    let redisInstance;
    try {
        redisInstance = (0, RedisHandler_1.ensureRedisInstance)(config.redisConnection, baseLogger);
        baseLogger.debug("Redis instance obtained/created.");
    }
    catch (redisError) {
        baseLogger.error({ err: redisError }, "Failed to obtain Redis instance.");
        throw redisError;
    }
    const notifiersToRegister = config.notifiers || {};
    const configuredNotifierKeys = Object.keys(notifiersToRegister);
    baseLogger.info({ configuredNotifiers: configuredNotifierKeys }, "Registering configured notifiers...");
    for (const [channel, options] of Object.entries(notifiersToRegister)) {
        if (!options) {
            baseLogger.trace({ channel }, "Skipping channel with no options provided.");
            continue;
        }
        const notifierConstructor = notifierMap[channel];
        if (notifierConstructor) {
            baseLogger.debug({ channel }, `Instantiating notifier...`);
            try {
                let notifierInstance;
                if (channel === "firebase") {
                    notifierInstance = new FirebaseNotifier_1.FirebaseNotifier(options);
                }
                else {
                    notifierInstance = new notifierConstructor(options);
                }
                baseLogger.debug({ channel }, "Registering notifier instance...");
                NotifierRegistry_1.NotifierRegistry.register(channel, notifierInstance);
            }
            catch (registrationError) {
                baseLogger.error({ err: registrationError, channel }, `Failed to instantiate or register notifier.`);
            }
        }
        else {
            baseLogger.warn({ channel }, `Unknown notifier type in configuration, skipping registration.`);
        }
    }
    const resolveTrackingKeyForJob = (job) => {
        return (job?.data?.trackingKey || config.trackingKey || "notifications:stats");
    };
    baseLogger.debug("Creating WorkerManager instance...");
    try {
        const workerManager = new WorkerManager_1.WorkerManager({
            redisConnection: redisInstance,
            queueName: config.queueName,
            concurrency: config.concurrency,
            trackingKey: config.trackingKey,
            bullWorkerOptions: config.bullWorkerOptions,
            onStart: config.onStart,
            onComplete: async (job, stats, jobLogger) => {
                jobLogger.debug({ stats }, 'WorkerManager "completed" event triggered.');
                const currentTrackingKey = resolveTrackingKeyForJob(job);
                if (config.onComplete) {
                    jobLogger.trace("Calling user onComplete callback...");
                    try {
                        await config.onComplete(job, stats, jobLogger);
                    }
                    catch (userCallbackError) {
                        jobLogger.error({ err: userCallbackError }, `Error in user-provided onComplete callback.`);
                    }
                }
                else {
                    jobLogger.info({ stats, trackingKey: currentTrackingKey }, "Job finished. Notification stats retrieved.");
                }
                if (config.resetStatsAfterCompletion) {
                    jobLogger.warn({ trackingKey: currentTrackingKey }, "Resetting notification stats as configured...");
                    try {
                        await (0, ResponseTrackers_1.resetNotificationStats)(redisInstance, currentTrackingKey, jobLogger);
                    }
                    catch (resetError) {
                        jobLogger.error({ err: resetError, trackingKey: currentTrackingKey }, "Failed to reset stats.");
                    }
                }
            },
            onDrained: config.onDrained,
        });
        baseLogger.info("WorkerManager instance created successfully. Worker server setup complete.");
        return workerManager;
    }
    catch (managerError) {
        baseLogger.error({ err: managerError }, "Failed to create WorkerManager instance.");
        throw managerError;
    }
}
//# sourceMappingURL=StartWorkerServer.js.map